@@ -6,7 +6,51 @@ use strict;
 use warnings;
 
 # Load the Perl pragma Exporter.
-use Exporter;
+use vars qw(@ISA @EXPORT @EXPORT_OK);
+use Exporter 'import';
+
+# Exporting the implemented subroutines.
+our @EXPORT = qw(
+    reset
+    clear
+    winsize
+    chars
+    pixels
+    get_cursor_position
+    set_cursor_position
+    echo_on
+    echo_off
+    cursor_on
+    cursor_off
+    cursor_up
+    cursor_down
+    cursor_forward
+    cursor_backward
+    which_terminal
+);
+
+# Import method on demand.
+our @EXPORT_OK = qw(
+    screen_size_pixels
+    screen_size_chars
+    window_size_chars
+    window_size_pixels
+    TermColor
+    TermName
+    TermPath
+    TermType
+    reset_screen
+    clear_screen
+    reset_terminal
+    clear_terminal
+    WinSize
+); 
+
+# Base class of this module.
+our @ISA = qw(Exporter);
+
+# Set the package version. 
+our $VERSION = '0.17';
 
 # Load the neccessary Perl modules.
 use bytes;
@@ -25,12 +69,12 @@ use Inline 'C' => <<'END_OF_C_CODE';
  */
 
 /*
- * Function: Shell()
- * -----------------
+ * Function: LoginShell()
+ * ----------------------
  */
-char* Shell() {
-    char *shell = getenv("SHELL");
-    return shell;
+char* LoginShell() {
+    char *login_shell = getenv("SHELL");
+    return login_shell;
 };
 
 /*
@@ -185,55 +229,6 @@ void WinSize() {
 
 END_OF_C_CODE
 
-# Set the package version. 
-our $VERSION = '0.16';
-
-# Base class of this module.
-our @ISA = qw(Exporter);
-
-# Exporting the implemented subroutines.
-our @EXPORT = qw(
-    reset_screen
-    clear_screen
-    reset_terminal
-    clear_terminal
-    reset
-    clear
-    winsize
-    chars
-    pixels
-    winsize_ioctl
-    winsize_stty
-    WinSize
-    screen_size_pixels
-    screen_size_chars
-    window_size_chars
-    window_size_pixels
-    get_cursor_position
-    set_cursor_position
-    echo_on
-    echo_off
-    cursor_on
-    cursor_off
-    ctlseqs_request
-    on_echo
-    off_echo
-    TermColor
-    TermName
-    TermPath
-    TermType
-    echo_icanon_on
-    echo_icanon_off
-    cursor_up
-    cursor_down
-    cursor_forward
-    cursor_backward
-    terminal
-    User
-    Logname
-    Shell
-);
-
 # Set the global variables.
 our($ESC, $CSI, $OSC);
 
@@ -460,9 +455,9 @@ sub ctlseqs_request {
         eval "$mod_alarm 0";
     };
     if ($@) {
-         # print "timedout\n";
+         # On timeout raise an exception.
          try {
-             # On error die. Raises an exception.
+             # On error die.
              die;
          } catch {
              # Clean up settings.  
@@ -685,27 +680,57 @@ sub cursor_backward {
 };
 
 #------------------------------------------------------------------------------# 
-# Subroutine inuse_terminal                                                    #
-# ps -o 'cmd=' -p $(ps -o 'ppid=' -p $$)                                       #
+# Subroutine trim                                                              #
+# trim removes white spaces from both ends of a string.                        #
 #------------------------------------------------------------------------------# 
-sub terminal {
-    # Declare local variables.
-    my $term = undef;
-    my $shell = undef;
+sub trim {
+    my $str = $_[0];
+    $str =~ s/^\s+|\s+$//g;
+    return $str;    
+};
+
+#------------------------------------------------------------------------------# 
+# Subroutine add_filter_chr                                                    #
+# Adds [ and ] to searchstring for grep                                        # 
+#------------------------------------------------------------------------------# 
+sub add_filter_chr {
+    my $str = $_[0];
+    substr($str, 0, 0) = '[';
+    substr($str, 2, 0) = ']';
+    return $str;
+};
+
+#------------------------------------------------------------------------------# 
+# Subroutine login_shell                                                       #
+#------------------------------------------------------------------------------# 
+sub login_shell {
+    my $user = $ENV{USER};
+    my $passwd = `cat /etc/passwd |grep -w $user`;
+    $passwd =~ s/^\s+|\s+$//g;
+    my ($shell) = $passwd =~ /^.*\/(.*)$/;
+    return $shell;
+};
+
+#------------------------------------------------------------------------------# 
+# Subroutine terminal                                                          # 
+# Terminal identification using bash:                                          #
+# ps -o 'cmd=' -p $(ps -o 'ppid=' -p $$)                                       #
+# my $tshell = LoginShell();                                                   # 
+# ($shell) = $tshell =~ /^.*\/(.*)$/;                                          #
+# cat /etc/shells                                                              #
+#------------------------------------------------------------------------------# 
+sub which_terminal {
     # Get the terminal path.
-    my $tpath = TermPath();
-    # Extract the pts from the terminal path.
-    ($term) = $tpath =~ /^.*\/dev\/(pts\/\d+)$/;
-    # Extract the shell from the shell path
-    my $tshell = Shell();
-    ($shell) = $tshell =~ /^.*\/(.*)$/;
-    # Add square brackets for use of grep.
-    substr($term, 0, 0) = '[';
-    substr($term, 2, 0) = ']';
-    substr($shell, 0, 0) = '[';
-    substr($shell, 2, 0) = ']';
+    my $termpath = TermPath();
+    # Extract the pts fd from the terminal path.
+    my ($ptsfd) = $termpath =~ /^.*\/dev\/(pts\/\d+)$/;
+    # Extract the login shell from the passwd file.
+    my $loginshell = login_shell();
+    # Add square brackets for use with grep.
+    $ptsfd = add_filter_chr($ptsfd);
+    $loginshell = add_filter_chr($loginshell);
     # Filter process by shell and pts.
-    my $process = `ps aux |grep $shell| grep $term`;
+    my $process = `ps aux |grep $loginshell| grep $ptsfd`;
     # Split up the process by lines and white spaces.
     my @lines = split /\n/, $process;
     my @columns = split /\s+/, $lines[0];
@@ -713,14 +738,14 @@ sub terminal {
     my $pid = $columns[1];
     # Get the PPID from the command ps.
     my $ppid = `ps -o 'ppid=' -p $pid`;
-    # Trim the PPID.  
-    $ppid =~ s/^\s+|\s+$//g;
-    # Get the terminal in use from the command ps.
-    my $this_term = `ps -o 'cmd=' -p $ppid`;
-    # Trim the terminal variable.  
-    $this_term =~ s/^\s+|\s+$//g;
-    # Return the terminal in use.
-    return $this_term;
+    # Trim the string withe the PPID.
+    $ppid = trim($ppid);
+    # Get the term in use from the command ps.
+    my $term = `ps -o 'cmd=' -p $ppid`;
+    # Trim the term variable.  
+    $term = trim($term);
+    # Return the term in use.
+    return $term;
 };
 
 1;
@@ -734,8 +759,16 @@ Terminal::Control - Perl extension with methods for the control of the terminal
 
 =head1 SYNOPSIS
 
+Use of standard methods:
+
   use Terminal::Control;
 
+or
+
+  use Terminal::Control qw(:DEFAULT);
+
+allows the import of the standard methods.
+
   # Clear or reset the terminal window.
 
   clear_screen();
@@ -747,17 +780,6 @@ Terminal::Control - Perl extension with methods for the control of the terminal
   ($rows, $cols) = chars();
   ($xpix, $ypix) = pixels();
 
-  # Get the terminal window size using Xterm control sequences. 
-
-  ($rows, $cols) = window_size_chars([$timeout]);  
-  ($xpix, $ypix) = windows_size_pixels([$timeout]);
-  ($rows, $cols) = screen_size_chars([$timeout]); 
-  ($xpix, $ypix) = screen_size_pixels([$timeout]);
-
-  # Get the terminal window size using C header 'sys/ioctl.h'.
-
-  ($rows, $cols, $xpix, $ypix) = Terminal::Control::WinSize();
-
   # Get and set the cursor position on the terminal window.
 
   ($row, $col) = get_cursor_position();
@@ -780,16 +802,60 @@ Terminal::Control - Perl extension with methods for the control of the terminal
   echo_on();
   echo_off();
 
+  # Get the terminal in use.
+
+  which_terminal();   
+
+C<qw(:DEFAULT)> imports all methods declared in C<@EXPORT> in the module.
+
+Use of optional methods:
+
+  use Terminal::Control qw(
+     :DEFAULT
+     window_size_chars window_size_pixels
+     screen_size_chars screen_size_pixels
+     TermColor TermType TermName TermPath
+     WinSize
+  );
+
+or
+
+  use Terminal::Control qw(
+     window_size_chars window_size_pixels
+     screen_size_chars screen_size_pixels
+     TermColor TermType TermName TermPath
+     WinSize
+  );
+
+allows the import of the optional and the standard methods. Without C<:DEFAULT>
+only optional methods are imported.
+
+  # Get the terminal window size using Xterm control sequences. 
+
+  ($rows, $cols) = window_size_chars([$timeout]);  
+  ($xpix, $ypix) = windows_size_pixels([$timeout]);
+  ($rows, $cols) = screen_size_chars([$timeout]); 
+  ($xpix, $ypix) = screen_size_pixels([$timeout]);
+
+  # Get the terminal window size using C header 'sys/ioctl.h'.
+
+  ($rows, $cols, $xpix, $ypix) = WinSize();
+
   # Get informations about the terminal.
 
-  Terminal::Control::TermColor();
-  Terminal::Control::TermType();  
-  Terminal::Control::TermName();  
-  Terminal::Control::TermPath();  
+  TermColor();
+  TermType();  
+  TermName();  
+  TermPath();
+
+The former methods are declared in C<@EXPORT_OK> in the module.
 
 Variables in square brackets in the method call are optional. They are in the
 related method predefined.
 
+In general every method is still accessible from outside the module using the
+syntax I<Terminal::Control::method_name>.
+
 =head1 DESCRIPTION
 
 =head2 Preface
@@ -807,11 +873,13 @@ basic methods are used to delete or reset a terminal window and to query the
 current window size. Other methods showed or suppressed the input echo. Showing
 or hiding the cursor is achieved by other methods.
 
-=head2 Methods implemented
+=head2 List of methods
 
 The following methods have been implemented so far within the module.
 The methods below are sorted according to their logical relationship.
 
+B<Standard methods>
+
 =over 4 
 
 =item * clear_screen()
@@ -822,19 +890,11 @@ The methods below are sorted according to their logical relationship.
 
 =over 4 
 
-=item * screen_size_chars($timeout) 
+=item * winsize()
 
-=item * screen_size_pixels($timeout)
+=item * chars()
 
-=item * window_size_chars($timeout)
-
-=item * windows_size_pixels($timeout)
-
-=back
-
-=over 4 
-
-=item * Terminal::Control::WinSize()
+=item * pixels()
 
 =back
 
@@ -848,11 +908,13 @@ The methods below are sorted according to their logical relationship.
 
 =over 4 
 
-=item * winsize()
+=item * cursor_up()
 
-=item * chars()
+=item * cursor_down()
 
-=item * pixels()
+=item * cursor_forward()
+
+=item * cursor_backward()
 
 =back
 
@@ -874,13 +936,27 @@ The methods below are sorted according to their logical relationship.
 
 =over 4 
 
-=item * cursor_up()
+=item * which_terminal()
 
-=item * cursor_down()
+=back
 
-=item * cursor_forward()
+B<Optional methods>
 
-=item * cursor_backward()
+=over 4 
+
+=item * WinSize()
+
+=back
+
+=over 4 
+
+=item * screen_size_chars($timeout) 
+
+=item * screen_size_pixels($timeout)
+
+=item * window_size_chars($timeout)
+
+=item * windows_size_pixels($timeout)
 
 =back
 
@@ -948,7 +1024,7 @@ B<screen_size_pixels([$timeout]);>
 Get the screen size in pixels. The methods gets the screen size in pixels using 
 Xterm control sequences.
 
-B<Terminal::Control::WinSize()>
+B<WinSize()>
 
 Get the window size in chars and pixels. The method is using in The Inline C code 
 the C-header for the system call C<ioctl> and the call or command C<TIOCGWINSZ>. The
@@ -986,35 +1062,43 @@ turning the echo of (user) input off.
 
 B<cursor_up()>
 
-Cursor up.
+Moves the cursor 1 char up.
 
 B<cursor_down()>
 
-Cursor down.
+Moves the Cursor 1 char down.
 
 B<cursor_forward()>
 
-Cursor forward.
+Moves the cursor 1 char forward.
 
 B<cursor_backward()>
 
-Cursor backward.
+Move the cursor 1 char backward.
 
 B<TermColor()>
 
-Color capabilities.
+Get the color capabilities of the terminal, e.g 24bit, truecolor.
 
 B<TermType()>
 
-Terminal capabilities.
+Get the terminal capabilities in general, e.g. xterm, xterm-color or xterm-256color.
 
 B<TermName()>
 
-Name of terminal.
+Get name of the terminal like '/dev/tty'.
 
 B<TermPath()>
 
-Path ot terminal.
+Get the path to the terminal like '/dev/pts/1'.
+
+B<which_terminal()>
+
+Get the name of the terminal emulator, which is in use by the user. First
+the method tries to fetch the login shell. Next the path to the terminal
+is determined. Based on this informations the related process to login shell
+and terminal path is identified. Evaluation of the PPID results in the parents
+process ID. The command related to this PID is the name of the terminal in use.   
 
 Variables in square brackets in the method call are optional. They are in the
 related method predefined.
@@ -1038,7 +1122,7 @@ size are as follows:
   # Reset the terminal window using ANSI escape sequences.
   reset_screen();
 
-=head2 Terminal window size
+=head2 Standard terminal window size
 
   # Declare the variables.
   my ($rows, $cols, $xpix, $ypix) = undef;
@@ -1055,6 +1139,17 @@ size are as follows:
   ($xpix, $ypix) = pixels();
   printf ("%s\n%s\n", $xpix, $ypix);
 
+=head2 Optional terminal window size
+
+  # Load the required module.
+  use Terminal::Control qw(
+     window_size_chars
+     window_size_pixels
+     screen_size_chars
+     screen_size_pixels
+     WinSize
+  );
+
   # Set the timeout for reading from STDIN in microseconds.
   my $timeout = 1000000;
 
@@ -1100,7 +1195,7 @@ size are as follows:
   # Disable echoing of commands using stty. 
   echo_off();
 
-=head2 Working example for cursor movement
+=head2 Cursor movement on the terminal window
 
 The script works as follows. The terminal window is reseted. Then the position
 of the cursor is set to (20, 40) on the terminal window. Now the user can use
@@ -1108,6 +1203,9 @@ the the arrow keys on the keyboard to move the cursor on the terminal window.
 Pressing 'q' quits the loop and set the cursor position to (1, 1) which is be
 the home position.
 
+  # Load the required module.
+  use Terminal::Control;
+
   # Initialise variable $chr.
   my $chr = '';
 
@@ -1190,17 +1288,6 @@ Using C<reset> on the command line and the Perl command C<system("reset")> have
 the same result in time of execution. Detailed comparison is outstanding. This one example
 already shows a significant advantage of the new routine.
 
-=head2 Remark on Xterm control sequences  
-
-The use of Xterm escape sequences is still considered experimental.
-
-Reading from STDIN is blocking with respect to the request of escape sequences,
-if no data are responded. To overcome the problem of waiting endless for an user
-input a timeout is programmed. 
-
-If the standard value is not applicable, reduce the timeout step by step. If the
-timeout is to short the method is not able to catch the input from STDIN. 
-
 =head1 SYSTEM COMPATIBILITY
 
 Three things must be fulfilled in order to use the full functionality of the
@@ -1306,12 +1393,23 @@ Terminals tested so far with the module:
 
 =head1 LIMITATIONS
 
+=head2 Xterm control sequences
+
+The use of Xterm escape sequences is still considered experimental.
+
+Reading the response from STDIN is blocking with respect to the request
+of escape sequences, if no data are responded. To overcome the problem of
+waiting endless for an user input a timeout is programmed. 
+
 If the timeout is set too short, the response to the request cannot be
 evaluated. If information on the size of the window is requested in quick
-succession, the data from STDIN will be read in incorrectly. This is only
-possible by deleting STDIN. A method of deleting or resetting STDIN is not 
+succession, the data from STDIN will be read in incorrectly. This can be
+overcome by deleting STDIN. A method of deleting or resetting STDIN is not 
 known yet.
 
+If the standard value is not applicable, reduce the timeout step by step.
+If the timeout is to short the method is not able to catch the input from STDIN. 
+
 =head1 OPEN ISSUES
 
 It is not yet clear how an installation of the module for Unix or Unix-like
